package com.yonge.cooleshow.admin.controller;


import com.yonge.cooleshow.biz.dal.dto.ImGroupSearchDto;
import com.yonge.cooleshow.biz.dal.entity.ImGroup;
import com.yonge.cooleshow.biz.dal.enums.ClientEnum;
import com.yonge.cooleshow.biz.dal.service.ImGroupService;
import com.yonge.cooleshow.biz.dal.service.SysUserService;
import com.yonge.cooleshow.common.controller.BaseController;
import com.yonge.cooleshow.common.entity.HttpResponseResult;
import com.yonge.toolset.utils.date.DateUtil;
import com.yonge.toolset.utils.validator.ValidationKit;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.http.HttpStatus;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;
import javax.validation.Valid;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;
import java.util.List;

/**
 * 即时通讯群组(ImGroup)表控制层
 *
 * @author zx
 * @since 2022-03-22 10:45:56
 */
@Api(tags = "即时通讯群组")
@RestController
@RequestMapping("/imGroup")
public class ImGroupController extends BaseController {
    /**
     * 服务对象
     */
    @Resource
    private ImGroupService imGroupService;
    @Resource
    private SysUserService sysUserService;

    @ApiOperation("获取群详情")
    @PostMapping(value = "/getDetail/{groupId}")
    @PreAuthorize("@pcs.hasPermissions('imGroup/detail')")
    public HttpResponseResult<ImGroup> getDetail(@ApiParam(value = "群编号", required = true) @PathVariable("groupId") String groupId) throws Exception {
        ImGroup group = imGroupService.getById(groupId);
        if (group == null) {
            return failed(HttpStatus.NO_CONTENT, "群组不存在");
        }
        return succeed(group);
    }

    @ApiOperation("群列表")
    @PostMapping(value = "/queryAll")
    @PreAuthorize("@pcs.hasPermissions('imGroup/queryAll')")
    public HttpResponseResult<List<ImGroup>> queryAll(@Valid @RequestBody ImGroupSearchDto imGroupSearchDto, BindingResult bindingResult) throws Exception {
        ValidationKit.ignoreFields(bindingResult,"createUserId");
        imGroupSearchDto.setUserId(sysUserService.getUserId());
        return succeed(imGroupService.queryAll(imGroupSearchDto));
    }

    @ApiOperation("退出群聊")
    @PostMapping(value = "/quit/{groupId}")
    public HttpResponseResult quit(@ApiParam(value = "群编号", required = true) @PathVariable("groupId") String groupId) throws Exception {
        imGroupService.quit(groupId,sysUserService.getUserId(), ClientEnum.STUDENT);
        return succeed();
    }



    @GetMapping(value = "/syncImHistoryMessageTask")
    // 融云同步即时通讯聊天记录
    public void syncImHistoryMessageTask(String date) throws Exception {
        if (date == null) {
            date = DateUtil.format(DateUtil.addHours(new Date(), -2), DateUtil.YEAR_MONTH_DAY_HOUR);
        }
        // 获取输入日期
        SimpleDateFormat format = new SimpleDateFormat("yyyyMMddHH");
        Date currentDate = format.parse(date);

        // 创建Calendar对象 设置为输入时间
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(currentDate);

        // 将日期减少三天
        calendar.add(Calendar.DATE, -3);

        // 获取减少三天后的日期
        Date targetTime = calendar.getTime();

        calendar.setTime(currentDate);

        //按照小时递减
        while (currentDate.after(targetTime)) {
            imGroupService.getAndSaveImHistoryMessage(DateUtil.format(currentDate, DateUtil.YEAR_MONTH_DAY_HOUR));
            calendar.add(Calendar.HOUR, -1);
            currentDate = calendar.getTime();
        }

    }


}

